@@ -98,7 +98,6 @@ public:
      * @return Use a @c Paginator if you wish to paginate through the results, otherwise returns the first page of results.
      */
     virtual QFuture<VideoListResult> requestFeed(Paginator * = nullptr) = 0;
-    virtual QFuture<VideoListResult> requestTop() = 0;
     /**
      * @brief Request the trending videos on this server, if available.
      * @note The trending topic may be ignored if this API does not support it. Check this feature by calling supportsFeature().
@@ -121,6 +120,7 @@ public:
     virtual QFuture<PlaylistsResult> requestChannelPlaylists(const QString &channelId) = 0;
     virtual QFuture<Result> addVideoToPlaylist(const QString &plid, const QString &videoId) = 0;
     virtual QFuture<Result> removeVideoFromPlaylist(const QString &plid, const QString &indexId) = 0;
+    virtual QString getVideoUrl(const QString &videoId) = 0;
 
 Q_SIGNALS:
     void credentialsChanged();
@@ -143,8 +143,16 @@ protected:
             reply->deleteLater();
         });
         connect(reply, &QNetworkReply::errorOccurred, this, [=](QNetworkReply::NetworkError error) {
-            qWarning() << "Error for" << error << reply->url() << reply->errorString();
-            interface->reportResult(std::pair(error, reply->errorString()));
+            QString errorString = reply->errorString();
+            // Prefer a JSON error string if provided
+            // FIXME: this invidious-specific
+            if (auto doc = QJsonDocument::fromJson(reply->readAll()); !doc.isNull()) {
+                if (doc.object().contains(QStringLiteral("error"))) {
+                    errorString = doc.object()[QStringLiteral("error")].toString();
+                }
+            }
+            qWarning() << "Error for" << error << reply->url() << errorString;
+            interface->reportResult(std::pair(error, errorString));
             interface->reportFinished();
             reply->deleteLater();
         });
